import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Astleigh.Chatterbug";

pluginVersion = "1.0";

pluginCommand = Turbine.ShellCommand();

local function out(msg)
	Turbine.Shell.WriteLine(msg);
end

function pluginCommand:Execute(cmd, args)
	if (args == "show") then
		mainWindow:SetVisible(true);
	elseif (args == "hide") then
		mainWindow:SetVisible(false);
	elseif (args == "toggle") then
		mainWindow:SetVisible(not mainWindow:IsVisible());
	else
		pluginCommand:GetHelp();
	end
end

function pluginCommand:GetHelp()
	out("Chatterbug v"..pluginVersion);
	out("  usage: /chatterbug [show|hide|toggle]");
end

Turbine.Shell.AddCommand("chatterbug", pluginCommand);

out("Chatterbug v"..pluginVersion.." (/chatterbug for more info)");

defaultSettings = {
    mainwindow = {
        position = {left = 100, top = 100}, 
        size = {width = 400, height = 400}
    },
    settingswindow = {
        position = {left = 100, top = 100},
    },
    showtimestamps = false,
    hidewindowonload = false,
    channels = {
        [Turbine.ChatType.Emote] = true,
        [Turbine.ChatType.Say] = true,
    }
};

function saveSettings()
	-- workarounds for savedata in locales that use comma for decimal point
	-- http://forums.lotro.com/showthread.php?365323-Turbine.PluginData.Save-in-german-%28and-french%29-version-of-client
    settings.version = pluginVersion;
	settings.mainwindow.position.left = tostring(mainWindow:GetLeft());
	settings.mainwindow.position.top = tostring(mainWindow:GetTop());
	settings.mainwindow.size.width = tostring(mainWindow:GetWidth());
	settings.mainwindow.size.height = tostring(mainWindow:GetHeight());
	settings.settingswindow.position.left = tostring(settingsWindow:GetLeft());
	settings.settingswindow.position.top = tostring(settingsWindow:GetTop());
    settings.showtimestamps = showtimestamps;
    settings.hidewindowonload = hidewindowonload;
    settings.channels = {};
    for channel,val in pairs(channels) do
        settings.channels[tostring(channel)] = val;
    end
	Turbine.PluginData.Save(Turbine.DataScope.Account, "Chatterbug", settings);
end

settings = Turbine.PluginData.Load(Turbine.DataScope.Account, "Chatterbug") or defaultSettings;

showtimestamps = settings.showtimestamps;
hidewindowonload = settings.hidewindowonload;
channels = {};
for channel,val in pairs(settings.channels) do
    channels[(channel + 0)] = val;
end
history = {};
filter = "";
lines = 0;
bytes = 0;

function memUsage()
    return "Buffer: " .. lines .. " lines/" .. string.format("%i", bytes/1024) .. "KB";
end

function padzero(n)
    if (n < 10) then
        return "0" .. n;
    else
        return n;
    end

end

function formatTimestamp(d)
    return "[" .. d.Hour .. ":" .. padzero(d.Minute) .. "]";
end

function format(msg)
    if (showtimestamps) then
        return formatTimestamp(msg.ts) .. " " .. msg.txt;
    else
        return msg.txt;
    end
end

function matchesFilter(msg)
    return channels[msg.ct] and (filter == "" or msg.txt:lower():find(filter,1,true) ~= nil);
end

function bool(x)
    if (x == nil) then
        return "f";
    else
        return "t";
    end
end

function chatHandler(sender, args)
    local len = args.Message:len();
    if (args.Message:sub(len,len) ~= '\n') then
        args.Message = args.Message .. '\n';
    end
    local msg = {ct = args.ChatType,
                 ts = Turbine.Engine.GetDate(),
                 txt = args.Message};
    table.insert(history, msg);
    lines = lines + 1;
    bytes = bytes + #msg.txt;
    settingsWindow.memusage:SetText(memUsage());
    if (matchesFilter(msg)) then
        mainWindow.text:AppendText(format(msg));
    end
end

-- try to play nicely with other plugins that use chat.received
AddCallback(Turbine.Chat, "Received", chatHandler);

optionsPanel = OptionsPanel();
plugin.GetOptionsPanel = function()
    return optionsPanel;
end

mainWindow = MainWindow();
if not hidewindowonload then
    mainWindow:SetVisible(true);
end

settingsWindow = SettingsWindow();
